"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const tty_table_1 = __importDefault(require("tty-table"));
const lodash_1 = require("lodash");
class CommentStore {
    constructor() {
        this.entries = {};
    }
    addComment(name, comment) {
        if (this.entries[name]) {
            this.entries[name].push(comment);
        }
        else {
            this.entries[name] = [comment];
        }
    }
    read(name) {
        return (this.entries[name] || []).join(', ');
    }
}
const analyze = (assets) => {
    const store = new CommentStore();
    const chunks = assets.filter(asset => asset.group === 'chunk');
    const averageChunkSize = lodash_1.mean(chunks.map(asset => asset.size));
    for (const { name, size, group } of assets) {
        // 1.5MB以上报警
        if (size > 1.5 * 1024 * 1024 && group !== 'map') {
            store.addComment(name, 'too large');
        }
        // 偏移超过40%的资源报警
        if (Math.abs(size - averageChunkSize) / averageChunkSize > 0.4 && group === 'chunk') {
            store.addComment(name, 'size unbalanced');
        }
    }
    return store;
};
exports.default = (assets) => {
    const colorize = (value, group) => {
        switch (group) {
            case 'chunk':
                return chalk_1.default.bold(value);
            case 'map':
                return chalk_1.default.gray(value);
            default:
                return value;
        }
    };
    const comments = analyze(assets);
    const headers = [
        {
            value: 'Name',
            align: 'left',
            width: (lodash_1.max(assets.map(asset => asset.name.length)) || 0) + 2,
            formatter({ name, group }) {
                return colorize(name, group);
            },
        },
        {
            value: 'Size',
            align: 'right',
            formatter({ size, group }) {
                const kb = (size / 1024).toFixed(2) + 'KB';
                return colorize(kb, group);
            },
        },
        {
            value: 'Comment',
            align: 'left',
            // 这里不能用`color`配置，似乎`yellowBright`这个颜色在`kleur`中没有，所以会出错
            formatter(value) {
                return chalk_1.default.yellowBright(value);
            },
        },
    ];
    const rows = assets.map(asset => [asset, asset, comments.read(asset.name)]);
    const table = new tty_table_1.default(headers, rows);
    console.log(table.render());
};
//# sourceMappingURL=report.js.map