"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = require("lodash");
const webpack_1 = __importDefault(require("webpack"));
const build_1 = require("../../../lib/build");
const project_1 = require("../../../lib/project");
const report_1 = __importDefault(require("./report"));
const GROUP_PRIORITY = {
    entry: 0,
    chunk: 1,
    asset: 2,
    map: 3,
};
const normalizeAssetWith = (entryNames, topLevelChunkNames) => {
    // SourceMap不算在入口中，开发者不需要太关注这东西的体积
    const chunksSet = new Set(topLevelChunkNames.filter(name => !name.endsWith('.map')));
    const isEntry = (filename) => {
        if (!filename.endsWith('.html')) {
            return false;
        }
        const isMatch = (filename, entryName) => {
            if (filename === entryName + '.html') {
                return true;
            }
            return filename.startsWith(entryName + '-');
        };
        return entryNames.some(entry => isMatch(filename, entry));
    };
    const getGroup = (asset) => {
        const name = path.basename(asset.name);
        if (chunksSet.has(name)) {
            return 'chunk';
        }
        if (isEntry(name)) {
            return 'entry';
        }
        if (name.endsWith('.map')) {
            return 'map';
        }
        return 'asset';
    };
    return (asset) => {
        const group = getGroup(asset);
        return {
            group,
            name: asset.name,
            size: asset.size,
        };
    };
};
const sortAsset = (x, y) => {
    const xPriority = GROUP_PRIORITY[x.group];
    const yPriority = GROUP_PRIORITY[y.group];
    if (xPriority === yPriority) {
        return x.name.localeCompare(y.name);
    }
    return xPriority - yPriority;
};
exports.default = async (cmd) => {
    const projectSettings = project_1.readProjectSettings(cmd, 'build');
    // 理论上，这个默认值是不会出现的，总不能`package.json`里连个`name`也不写
    const { name: hostPackageName } = project_1.readHostPackageConfig(cmd.cwd);
    const env = {
        hostPackageName,
        usage: 'build',
        mode: cmd.mode,
        cwd: cmd.cwd,
        srcDirectory: cmd.src,
        buildTarget: cmd.buildTarget,
        analyze: cmd.analyze || false,
        ...projectSettings,
    };
    const configurations = build_1.createBuildConfig(env);
    process.env.NODE_ENV = env.mode;
    if (cmd.clean) {
        rimraf_1.default.sync(path.join(env.cwd, 'dist'));
    }
    webpack_1.default(configurations, // https://github.com/Microsoft/TypeScript/issues/14107
    (err, stats) => {
        if (err) {
            console.error(err);
            process.exit(1);
        }
        const toJsonOptions = { all: false, assets: true };
        // webpack的`toJson`的定义是错的
        const { errors, warnings, children } = stats.toJson(toJsonOptions);
        for (const error of lodash_1.uniq(errors)) {
            console.log('');
            console.error(chalk_1.default.redBright(chalk_1.default.bold(error)));
        }
        for (const warning of lodash_1.uniq(warnings)) {
            console.log('');
            console.warn(chalk_1.default.yellowBright(chalk_1.default.bold(warning)));
        }
        const entries = lodash_1.uniq(lodash_1.flatMap(configurations, config => Object.keys(config.entry || {})));
        const chunkNames = lodash_1.uniq(lodash_1.flatMap(children, child => child.assets.map(asset => asset.name)));
        const normalize = normalizeAssetWith(entries, chunkNames);
        const assets = lodash_1.uniqBy(lodash_1.flatMap(children, child => child.assets.map(normalize)), asset => asset.name);
        const sortedAssets = assets.sort(sortAsset);
        report_1.default(sortedAssets);
        if (stats.hasErrors()) {
            process.exit(1);
        }
    });
};
//# sourceMappingURL=run.js.map