"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const child_process_1 = require("child_process");
const webpack_merge_1 = __importDefault(require("webpack-merge"));
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = require("lodash");
const project_1 = require("../../project");
exports.fillBuildSettings = (settings, cwd = process.cwd()) => {
    return {
        thirdParty: false,
        reportLintErrors: true,
        extractCSS: false,
        noCompileScripts: [],
        noModulesStyles: [],
        styleResources: [],
        largeAssetSize: 8 * 1024,
        browserSupport: { chrome: 60, safari: 11 },
        styleName: false,
        extraLessPaths: [],
        polyfill: true,
        copies: [],
        appTitle: (settings === null || settings === void 0 ? void 0 : settings.appTitle) || project_1.readHostPackageConfig(cwd).name,
        excludeFeatures: ['dev'],
        extraLessVariables: {},
        autoRouteSplit: false,
        ...settings,
    };
};
const fillDevServerSettings = (settings) => {
    return {
        https: false,
        port: 8788,
        apiPrefixes: [],
        defaultProxyDomain: '',
        hot: 'all',
        openPage: '',
        onStart: lodash_1.noop,
        ...settings,
    };
};
const fillRollupSettings = (settings) => {
    return {
        namedDependencyExports: {},
        styleOutput: 'index',
        target: 'universal',
        ...settings,
    };
};
const fillTestSettings = (settings) => {
    return {
        collectCoverageFrom: [],
        coveragePathIgnorePatterns: [],
        ...settings,
    };
};
const fillPlaySettings = (settings) => {
    return {
        extraResources: [],
        ...settings,
    };
};
exports.fillProjectSettings = (settings) => {
    const defaultAddition = () => ({});
    return {
        cwd: settings.cwd || process.cwd(),
        featureMatrix: settings.featureMatrix || { stable: {}, dev: {} },
        build: exports.fillBuildSettings(settings.build),
        devServer: fillDevServerSettings(settings.devServer),
        addition: settings.addition || defaultAddition,
        rollup: fillRollupSettings(settings.rollup),
        test: fillTestSettings(settings.test),
        play: fillPlaySettings(settings.play),
    };
};
const mergeBuiltin = webpack_merge_1.default.strategy({ 'module.rules': 'prepend' });
const mergeCustom = webpack_merge_1.default.strategy({ 'module.rules': 'replace' });
const revision = () => {
    try {
        return child_process_1.execSync('git rev-parse --short HEAD', { stdio: 'pipe' }).toString().trim();
    }
    catch (ex) {
        console.log('Not a git repository, fallback to default revision');
        return '0000000';
    }
};
const readEntryConfig = (name, dir) => {
    try {
        const config = require(path.join(dir, name + '.config'));
        return config;
    }
    catch (ex) {
        return {};
    }
};
const resolveEntryTemplate = (name, dir) => {
    const filename = path.join(dir, name + '.ejs');
    return fs.existsSync(filename) ? filename : null;
};
const collectEntries = (cwd, srcDirectory) => {
    const entriesFolder = path.join(cwd, srcDirectory, 'entries');
    if (!fs.existsSync(entriesFolder)) {
        console.error(chalk_1.default.red(`No ${srcDirectory}/entries dir found`));
        process.exit(1);
    }
    const entryScriptFiles = fs.readdirSync(entriesFolder)
        .filter(f => /\.[jt]sx?$/.test(f) && !f.includes('.config.'))
        .map(f => path.resolve(entriesFolder, f));
    const toEntry = (file) => {
        const name = path.basename(file, path.extname(file));
        const template = resolveEntryTemplate(name, entriesFolder);
        const config = readEntryConfig(name, entriesFolder);
        return { name, file, config, template };
    };
    return entryScriptFiles.map(toEntry);
};
exports.createBuildEnv = ({ build, ...sourceEnv }) => {
    const now = new Date();
    const entries = sourceEnv.usage === 'play' ? [] : collectEntries(sourceEnv.cwd, sourceEnv.srcDirectory);
    const buildEnv = {
        ...sourceEnv,
        entries,
        build: exports.fillBuildSettings(build),
        buildVersion: revision(),
        buildTime: now.toISOString(),
    };
    if (!buildEnv.featureMatrix) {
        console.error(chalk_1.default.yellowBright('Require featureMatrix in settings.js'));
        process.exit(1);
    }
    return buildEnv;
};
const toStringTag = Object.prototype.toString;
const getSchema = (obj) => {
    const entries = Object.entries(obj);
    entries.sort(([keyX], [keyY]) => keyX.localeCompare(keyY));
    return entries.map(([key, value]) => [key, toStringTag.call(value)]);
};
exports.checkFeatureMatrixSchema = (features) => {
    const featurePairs = Object.entries(features);
    // 计算每一个feature的结构是不是一样，如果有结构不一样的，则直接报错退出
    const baseSchema = getSchema(featurePairs[0][1]);
    const conflicts = featurePairs.slice(1).reduce((errors, [key, value]) => {
        const currentSchema = getSchema(value);
        if (!lodash_1.isEqual(baseSchema, currentSchema)) {
            return [...errors, key];
        }
        return errors;
    }, []);
    if (conflicts.length > 0) {
        throw new Error(`Build target ${conflicts.join(' & ')} have incompatible feature schema`);
    }
};
exports.checkPreCommitHookWhenLintDisabled = (cwd) => {
    const packageConfig = JSON.parse(fs.readFileSync(path.join(cwd, 'package.json'), 'utf-8'));
    if (!lodash_1.get(packageConfig, ['husky', 'hooks', 'pre-commit'])) {
        const warning = fs.readFileSync(path.join(__dirname, '../../../assets/lint-disabled-warning.txt'), 'utf-8');
        console.warn(chalk_1.default.yellowBright(warning));
        process.exit(1);
    }
};
exports.createDevEntry = (env) => {
    const buildEnv = exports.createBuildEnv(env);
    const { featureMatrix } = buildEnv;
    const buildTarget = buildEnv.buildTarget || 'default';
    const currentFeatures = featureMatrix[buildTarget];
    return {
        ...buildEnv,
        buildTarget,
        features: currentFeatures,
        isDefaultTarget: true,
    };
};
const resolveConfigurations = (entry, names) => {
    const importPartial = (name) => {
        const module = `../partials/${name}`;
        const { default: value } = require(module);
        return value;
    };
    const partials = names.map(importPartial);
    const configurations = partials.map(create => create(entry));
    return configurations;
};
exports.createWebpackConfig = (entry, extras) => {
    const { mode, addition } = entry;
    const partialNames = ['base', mode, ...extras];
    const configurations = resolveConfigurations(entry, partialNames);
    return mergeCustom(mergeBuiltin(...configurations), addition(entry));
};
exports.printableValue = (value) => {
    if (typeof value === 'boolean') {
        return value ? chalk_1.default.green('✓') : chalk_1.default.red('✕');
    }
    return value.toString();
};
//# sourceMappingURL=utils.js.map