"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const lodash_1 = require("lodash");
const getDefaultImportLocalName = (node) => {
    if (!node) {
        return null;
    }
    return lodash_1.get(node.specifiers.find(n => n.type === 'ImportDefaultSpecifier' || n.type === 'ImportNamespaceSpecifier'), ['local', 'name'], null);
};
const getNamedImportLocalName = (importedName, node) => {
    if (!node) {
        return null;
    }
    return lodash_1.get(node.specifiers.find(n => n.type === 'ImportSpecifier' && n.imported.name === importedName), ['local', 'name'], null);
};
const REACT_BASE_CLASS_NAMES = ['Component', 'PureComponent'];
const findReactImportNames = (path) => {
    const program = path.parentPath.find(n => n.type === 'Program');
    const imports = program.node.body.filter(n => n.type === 'ImportDeclaration');
    // 是有可能存在多个`react`的引入的，特别是有类似`babel-plugin-react-require`这种插件增加代码的情况下
    const reactImports = imports.filter(n => n.source.value === 'react');
    const defaults = lodash_1.compact(reactImports.map(getDefaultImportLocalName));
    const named = lodash_1.compact(lodash_1.flatMap(reactImports, importNode => REACT_BASE_CLASS_NAMES.map(name => getNamedImportLocalName(name, importNode))));
    return {
        defaults: new Set(defaults),
        named: new Set(named),
    };
};
exports.isExpressionContextElligible = ({ node, parentPath }) => {
    if (node.type === 'ArrowFunctionExpression' && parentPath.node.type === 'ExportDefaultDeclaration') {
        return true;
    }
    if (parentPath.node.type === 'VariableDeclarator') {
        return true;
    }
    if (parentPath.node.type !== 'AssignmentExpression') {
        return false;
    }
    return parentPath.parentPath.node.type === 'ExpressionStatement';
};
exports.isClassDefinitionElligible = (path) => {
    const { node: { superClass } } = path;
    if (!superClass) {
        return false;
    }
    const reactImportNames = findReactImportNames(path);
    if (superClass.type === 'Identifier') {
        return reactImportNames.named.has(superClass.name);
    }
    if (superClass.type === 'MemberExpression') {
        const object = superClass.object;
        const objectName = object.name;
        const propertyName = superClass.property.name;
        return reactImportNames.defaults.has(objectName) && REACT_BASE_CLASS_NAMES.includes(propertyName);
    }
    return false;
};
const validExpressionHead = (path) => {
    return path.isAssignmentExpression() || path.isVariableDeclarator() || path.isStatement();
};
// eslint-disable-next-line complexity
const resolvePossibleComponentName = (path, filename) => {
    const nodeWithID = path.node;
    if (nodeWithID.id) {
        return ['named', nodeWithID.id.name];
    }
    // 函数和类声明一定是有名字的，没名字的只能是表达式，有几种情况：
    //
    // 1. 变量声明，取变量名
    // 2. 对象赋值，取被赋值的对象或属性的名称
    // 3. 默认导出匿名函数或类，按照文件路径来猜名字
    if (path.parentPath.node.type === 'ExportDefaultDeclaration') {
        /* istanbul ignore next */
        if (!filename) {
            return ['module', 'Unknown'];
        }
        const file = path_1.basename(filename, path_1.extname(filename));
        return [
            'module',
            file === 'index' ? lodash_1.last(path_1.dirname(filename).split(path_1.sep)) : file,
        ];
    }
    const validHead = path.find(validExpressionHead);
    /* istanbul ignore next */
    if (!validHead) {
        return ['error', null];
    }
    if (validHead.isAssignmentExpression()) {
        const { node: { left } } = validHead;
        switch (left.type) {
            case 'Identifier':
                return ['assignment', left.name];
            case 'MemberExpression':
                return ['assignment', left.property.name];
            /* istanbul ignore next */
            default:
                throw new Error(`Unexpected assignment expression in ${filename}`);
        }
    }
    /* istanbul ignore else */
    if (validHead.isVariableDeclarator() && validHead.node.id.type === 'Identifier') {
        return ['declare', validHead.node.id.name];
    }
    /* istanbul ignore next */
    return ['error', null];
};
exports.isFunctionDefinitionElligible = (path, filename) => {
    const { node: { params } } = path;
    // 组件最多只能有1个参数
    if (params.length > 1) {
        return false;
    }
    const [nameType, functionName] = resolvePossibleComponentName(path, filename);
    return nameType !== 'error' && /^[A-Z]/.test(functionName);
};
// eslint-disable-next-line complexity
const computeDisplayNameAssignmentLeft = (node, path, filename) => {
    if (node.type === 'VariableDeclaration') {
        return node.declarations[0].id;
    }
    if (node.type === 'FunctionDeclaration' || node.type === 'ClassDeclaration') {
        return node.id;
    }
    if (node.type === 'ExpressionStatement' && node.expression.type === 'AssignmentExpression') {
        return node.expression.left;
    }
    if (node.type === 'ExportNamedDeclaration' && node.declaration) {
        return computeDisplayNameAssignmentLeft(node.declaration, path, filename);
    }
    if (node.type === 'ExportDefaultDeclaration' && node.declaration.type === 'Identifier') {
        return node.declaration;
    }
    const nodeWithID = path.node;
    /* istanbul ignore else */
    if (nodeWithID.id) {
        return nodeWithID.id;
    }
    /* istanbul ignore next */
    throw new Error(`Unexpected block statement of type ${node.type} in ${filename}`);
};
const replaceDefaultExportWithAssignable = (path, types, name) => {
    if (path.node.type === 'ArrowFunctionExpression') {
        // export default () => {}
        // 由于默认导出的箭头函数不能加名称，所以要先变成一个变量定义再导出去
        const autoNameIdentifier = path.parentPath.scope.generateUidIdentifier(name);
        const variableDeclaration = types.variableDeclaration('const', [
            types.variableDeclarator(autoNameIdentifier, types.cloneDeep(path.node)),
        ]);
        path.parentPath.insertBefore(variableDeclaration);
        path.parentPath.replaceWith(types.exportDefaultDeclaration(autoNameIdentifier));
        return path.parentPath;
    }
    // export default function () {}
    // export default class {}
    const id = path.scope.generateUidIdentifier(name);
    const nodeWithID = path.node;
    nodeWithID.id = id;
    return path;
};
exports.insertDisplayNameAfter = (path, types, filename) => {
    const [nameType, componentName] = resolvePossibleComponentName(path, filename);
    /* istanbul ignore next */
    if (nameType === 'error') {
        throw new Error(`Cannot resolve component display name from node of type ${path.node.type}`);
    }
    const exportDeclarationPath = nameType === 'module'
        ? replaceDefaultExportWithAssignable(path, types, componentName)
        : path;
    const blockStatement = exportDeclarationPath.find(path => path.parentPath.isBlock());
    /* istanbul ignore next */
    if (!blockStatement) {
        return;
    }
    const { expressionStatement, assignmentExpression, memberExpression, identifier, stringLiteral } = types;
    const displayNameStatement = expressionStatement(assignmentExpression('=', memberExpression(computeDisplayNameAssignmentLeft(blockStatement.node, path, filename), identifier('displayName')), stringLiteral(componentName)));
    blockStatement.insertAfter(displayNameStatement);
};
//# sourceMappingURL=utils.js.map